package travel.iko.sso.domain.security.service;

import lombok.RequiredArgsConstructor;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.server.authorization.OAuth2AuthorizationConsent;
import org.springframework.security.oauth2.server.authorization.OAuth2AuthorizationConsentService;
import org.springframework.security.oauth2.server.authorization.client.RegisteredClient;
import org.springframework.security.oauth2.server.authorization.client.RegisteredClientRepository;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import travel.iko.sso.domain.security.repository.AuthorizationConsentRepository;

/**
 * Created on: 9/14/21.
 *
 * @author Bjorn Harvold Responsibility:
 */
@RequiredArgsConstructor
//@Service("oAuth2AuthorizationConsentService")
public class MongoOAuth2AuthorizationConsentService implements OAuth2AuthorizationConsentService {
    private final AuthorizationConsentRepository authorizationConsentRepository;
    private final RegisteredClientRepository registeredClientRepository;

    @Override
    public void save(OAuth2AuthorizationConsent authorizationConsent) {
        final travel.iko.sso.domain.security.document.OAuth2AuthorizationConsent toSave = new travel.iko.sso.domain.security.document.OAuth2AuthorizationConsent(
                authorizationConsent.getRegisteredClientId(),
                authorizationConsent.getRegisteredClientId(),
                StringUtils.collectionToDelimitedString(authorizationConsent.getAuthorities(), ",")
        );

        this.authorizationConsentRepository.save(toSave);
    }

    @Override
    public void remove(OAuth2AuthorizationConsent authorizationConsent) {
        travel.iko.sso.domain.security.document.OAuth2AuthorizationConsent consent = this.authorizationConsentRepository.findByRegisteredClientIdAndPrincipalName(authorizationConsent.getRegisteredClientId(), authorizationConsent.getPrincipalName()).orElse(null);

        if (consent != null) {
            this.authorizationConsentRepository.delete(consent);
        }
    }

    @Override
    public OAuth2AuthorizationConsent findById(String registeredClientId, String principalName) {
        travel.iko.sso.domain.security.document.OAuth2AuthorizationConsent consent = this.authorizationConsentRepository.findByRegisteredClientIdAndPrincipalName(registeredClientId, principalName).orElse(null);

        return buildAuthorizationConsent(consent);
    }

    private OAuth2AuthorizationConsent buildAuthorizationConsent(travel.iko.sso.domain.security.document.OAuth2AuthorizationConsent consent) {
        if (consent == null) {
            return null;
        }

        final String registeredClientId = consent.getRegisteredClientId();
        final RegisteredClient registeredClient = this.registeredClientRepository.findById(registeredClientId);
        if (registeredClient == null) {
            throw new DataRetrievalFailureException(
                    "The RegisteredClient with id '" + registeredClientId + "' was not found in the RegisteredClientRepository.");
        }

        final String principalName = consent.getPrincipalName();

        OAuth2AuthorizationConsent.Builder builder = OAuth2AuthorizationConsent.withId(registeredClientId, principalName);
        String authorizationConsentAuthorities = consent.getAuthorities();
        if (authorizationConsentAuthorities != null) {
            for (String authority : StringUtils.commaDelimitedListToSet(authorizationConsentAuthorities)) {
                builder.authority(new SimpleGrantedAuthority(authority));
            }
        }
        return builder.build();
    }
}
